%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Authors: A. Ali and T. Y. Al-Naffouri                                           
% Last Modified: June, 2016
%
% If you use this code or any (modified) part of it in any publication, please cite 
% the paper: A. Ali, M. Masood, M. S. Sohail, S. Al-Ghadhban, and T. Y. Al-Naffouri, Narrowband Interference Mitigation in SC-FDMA Using Bayesian Sparse Recovery, IEEE Transactions on Signal Processing.
%
% Copyright (c) A. Ali, M. Masood, M. S. Sohail, S. Al-Ghadhban, and T. Y. Al-Naffouri
%
% Contact Persons: A. Ali and T. Y. Al-Naffouri
% E-mail: anumali@utexas.edu, tareq.alnaffouri@kaust.edu.sa
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clc;
clear all;
close all;
%
N=512;
%
j=sqrt(-1);
%
FN=1/sqrt(N)*dftmtx(N);
FHN=FN';
%
HN=haarmtx(N);
HHN=HN';
%
H_w=FN*diag(hamming(N))*FHN;
H_w=H_w/norm(H_w(:,1));
H_w_inv=inv(H_w);
%
max_no_NBI=6;
%
iter=1e3;
%
sparsity_true_vec=zeros(max_no_NBI,1);
sparsity_spd_vec=zeros(max_no_NBI,1);
sparsity_haar_vec=zeros(max_no_NBI,1);
sparsity_window_vec=zeros(max_no_NBI,1);
%
for dd=1:max_no_NBI
no_of_NBI=dd
%
sparsity_true=0;
sparsity_spd=0;
sparsity_haar=0;
sparsity_window=0;
%
prod_vec=(-(0:N-1)-0.5+N)/N;
%
for jj=1:iter
NBI_loc= randi([1,N],[no_of_NBI,1]) + -0.5 +rand(no_of_NBI,1);
NBI_amp=(randn(no_of_NBI,1)+j*randn(no_of_NBI,1));
NBI_p1_true=zeros(N,1);
NBI_p1_true(round(NBI_loc))=NBI_amp;
NBI_p1_spd=FN*exp(j*2*pi*(0:N-1)'*NBI_loc'/N)/sqrt(N)*NBI_amp;
NBI_p1_haar=HN*NBI_p1_spd;
NBI_p1_window=H_w*NBI_p1_spd;
%
NBI_p1_true_std=sort(abs(NBI_p1_true));
NBI_p1_spd_std=sort(abs(NBI_p1_spd));
NBI_p1_haar_std=sort(abs(NBI_p1_haar));
NBI_p1_window_std=sort(abs(NBI_p1_window));
%
sparsity_true=sparsity_true+(1-2/norm(NBI_p1_true,1)*prod_vec*NBI_p1_true_std);
sparsity_spd=sparsity_spd+(1-2/norm(NBI_p1_spd,1)*prod_vec*NBI_p1_spd_std);
sparsity_haar=sparsity_haar+(1-2/norm(NBI_p1_haar,1)*prod_vec*NBI_p1_haar_std);
sparsity_window=sparsity_window+(1-2/norm(NBI_p1_window,1)*prod_vec*NBI_p1_window_std);
end
sparsity_true_vec(dd)=sparsity_true/iter;
sparsity_spd_vec(dd)=sparsity_spd/iter;
sparsity_haar_vec(dd)=sparsity_haar/iter;
sparsity_window_vec(dd)=sparsity_window/iter;
end
%
figure,plot(1:max_no_NBI,sparsity_true_vec,'ro-');
hold on,plot(1:max_no_NBI,sparsity_spd_vec,'go-');
hold on,plot(1:max_no_NBI,sparsity_haar_vec,'mx-');
hold on,plot(1:max_no_NBI,sparsity_window_vec,'kp-');
%
legend('Original Signal (Perfect Alignment)','Spread Signal',...
    'Haar Transform Sparsification','Window Sparsification');
xlabel('No of NBI sources'),ylabel('Gini Index')